clear 
close all
clc

%% PARAMETERS

% polynomial functions for battery voltage, resistance and EM efficiency
% defined in the polynimial_fits.m file, and the funcions are saved
% separately to be used directly inside this model

addpath("polynomial_fits")
addpath("data\Eff_map")
addpath("data")

% simulation parameters
MPC.g = 9.81; % Gravity acceleration [m/s^2]
MPC.alpha = 0*pi/180; % road slope [rad]
MPC.rho = 1.25; % air density [kg/m^3] - from Onori HEV book

% battery parameters
MPC.N_s = 108; % Number of series
MPC.N_p = 1; % Number of parallels
MPC.N_b = MPC.N_s*MPC.N_p; % number of battery cells
MPC.Q_nom = 60*MPC.N_p; % nominal battery capacity [Ah]
MPC.eta_c = 0.95; % Coloumbic efficiency
load('bat_Ro_vs_SOC_data.mat'); % Ro variation with SOC - single battery cell
load('bat_Voc_vs_SOC_data.mat'); % Voc variation with SOC - single battery cell
MPC.F_Voc_s = griddedInterpolant(SOC_Voc_data(:,1),SOC_Voc_data(:,2)); % interpolated Voc data for a single cell
MPC.F_Ro_s = griddedInterpolant(SOC_Ro_data(:,1),SOC_Ro_data(:,2)); % interpolated Ro data for a single cell
MPC.SOC_vec = linspace(0,1,500);
MPC.Voc_s = MPC.F_Voc_s(MPC.SOC_vec); % vector with interpolated values of Voc_s (single cell)
MPC.Ro_s = MPC.F_Ro_s(MPC.SOC_vec); % vector with interpolated values of Ro_s (single cell)

% vehicle parameters
MPC.M_veh = 1400; % vehicle mass [kg]
MPC.a = 1; % Front axle - CoG  Front axle - CoG  [m]
MPC.b = 1.3; % Rear axle - CoG  [m]
MPC.h = 0.3; % Height of CoG [m]
MPC.f_0 = 4.5*1e-3; % Static rolling coefficient [N/kN]
MPC.k = 0; % miscellaneous loss coeff [Ns/m]
MPC.r_w = 0.3; % Wheel radius [m]
MPC.C_d = 0.33; % Drag coeff
MPC.A_f = 2.15; % Frontal area [m^2]
MPC.tau_gb = 9.6; % Gear ratio
MPC.eta_gb = 0.97; % Gearbox efficiency
MPC.eta_inv = 1; % Inverter efficiency

% electric motor parameters
load("Eff_map\Efficiency.mat"); % loading the EM efficeincy map
load("Eff_map\Shaft_torque.mat"); % loading shaft torque map
load("Eff_map\Speed.mat"); % loading speed map
MPC.Speed_max = Speed(:,1);
MPC.Torque_max = Shaft_Torque(:,1);
MPC.Efficiency(Efficiency == 0) = 10;
MPC.eff = Efficiency/100;

%% MPC

% PARAMETERS
MPC.Ts = 0.05; % Sampling time

MPC.par.nx         = 3; % number of states
MPC.par.nu         = 1; % control elements number
MPC.par.ny         = 3; % number of outputs
MPC.par.model = @prediction_longitudinal_model; %Modello di predizione
MPC.par.ub         = 250; % Upper bound saturazione input -> maximum value for control output 
MPC.par.lb         = -250; % Lower bound saturazione input -> minimum value for control output 
MPC.par.tol        = 1; % Reference tolerance
MPC.par.Nfev       = 150; % Interation number of fmincon in cost function (default 200)
MPC.par.Ts         = MPC.Ts;


% initial conditions
MPC.SOC0 = 0.8;
MPC.x0 = 0;
MPC.v0 = 0;
MPC.xx0 = [MPC.SOC0; MPC.x0; MPC.v0];

% Position reference parameters
MPC.par.R = 0.01; % matrice diagonale definita positiva per cost function
MPC.par.P = diag([0;50000;0]); % matrice diagonale definita positiva per cost function
MPC.par.Q = diag([0;1;0]); % matrice diagonale definita positiva per cost function
MPC.par.Tp = 10*MPC.Ts; % Prediction horizon (sempre multiplo intero del Ts)
MPC.K = nmpc_design_4b(MPC.par); %Generazione parametri design NMPC    
