clear;
clc;

% testing polinomial fit with battery and EM efficiency data

%% adding folders to path
addpath('data\Eff_map');
addpath('data');

%% loading data into vectors

%%% loading EM parameters
load('Efficiency.mat'); % loading the EM efficeincy map
load("Shaft_torque.mat"); % loading shaft torque map
load("Speed.mat"); % loading speed map 

%%% loading battery params
load("bat_Ro_vs_SOC_data.mat");
load("bat_Voc_vs_SOC_data.mat");

%% EM efficiency polynomial polynomial fitting and testing

% EM efficiency ref (compare the results with the polinomial)
eff_flat = Efficiency(:)/100;
torq_flat = Shaft_Torque(:);
speed_flat = Speed(:);

F = scatteredInterpolant(Speed(:),Shaft_Torque(:),Efficiency(:)/100);
figure
surf(Speed,Shaft_Torque,Efficiency);

speed_points = [500;750;1000;1250;1500;1750;2000;2250;2500;2750;3000;3250;3500;3750;4000;4250;4500;4750;5000;5250;5500;5750;6000;6250;6500;6750;7000;7250;7500;7750;8000;8250;8500;8750;9000;9250;9500;9750;10000];
torque_points = [287.315725265347;287.233717032146;287.151724602149;287.069743739971;286.987771986552;286.905807734913;286.823849853203;286.741897501612;267.378269069311;243.071146629631;222.814866998334;205.674858958375;190.983337167297;178.250632202600;167.109517497491;157.278993072683;148.540714541255;140.726295202330;133.690149986783;127.323954482924;121.536502005985;116.252306265388;111.408460170112;106.952121758109;102.838578612008;99.0297423697019;95.4928054975261;92.1999538366269;89.1266218638583;86.2515686389848;83.5562072631545;81.0242030664065;78.6411424126642;76.3942584067584;74.2722022887362;72.2648518732074;70.3631501887848;68.5589689496418;66.8449924891107];

%%% testing and comparing the results of polynimial fit and interpolant 

poly = [];
eval = [];
number_points = [];
for i=1:length(speed_points)
    poly = [poly; eff_poly(speed_points(i),torque_points(i))];
    eval = [eval; F(speed_points(i),torque_points(i))];
    number_points = [number_points;i];
end

% plotting the diferentece between the evaluations

figure
plot(number_points,poly - eval, 'LineWidth',1.5);
title('Difference from polynomial fit and interpolant evaluation of EM efficiency');
xlabel('Evaluations');
ylabel('Difference');

%% Battery voltage and resistance polynomial fitting and testing

% interpolants for comparison and tests
F_Voc_s = griddedInterpolant(SOC_Voc_data(:,1),SOC_Voc_data(:,2)); % interpolated Voc data for a single cell
F_Ro_s = griddedInterpolant(SOC_Ro_data(:,1),SOC_Ro_data(:,2)); % interpolated Ro data for a single cell

% Extracting data from files
SOC_R = SOC_Ro_data(:,1); 
R_data = SOC_Ro_data(:,2);

SOC_V = SOC_Voc_data(:,1);
V_data = SOC_Voc_data(:,2);

% Perform a 2nd-degree polynomial fit
degree = 2;
R_coefficients = polyfit(SOC_R, R_data, degree);
V_coefficients = polyfit(SOC_V, V_data, degree);

% Polynomial functions
R_function = poly2str(R_coefficients, 'SOC');
V_function = poly2str(V_coefficients, 'SOC');
disp(['Polynomial Equation: R(SOC) = ' R_function]);
disp(['Polynomial Equation: V(SOC) = ' V_function]);

% Create a polynomial model using polyval
SOC_range_R = linspace(min(SOC_R), max(SOC_R), 100); % Adjust the range as needed
Resistance_fit = polyval(R_coefficients, SOC_range_R);

SOC_range_V = linspace(min(SOC_V), max(SOC_V), 100); % Adjust the range as needed
Voltage_fit = polyval(V_coefficients, SOC_range_V);

% Plot the original data points and the fitted curve
figure;
plot(SOC_R, R_data, 'o', 'DisplayName', 'Original Data');
hold on;
plot(SOC_range_R, Resistance_fit, 'r-', 'DisplayName', '2nd-degree Polynomial Fit');
xlabel('State of Charge (SOC)');
ylabel('Resistance');
title('2nd-degree Polynomial Fit: Resistance as a function of SOC');
legend('Location', 'Best');
grid on;


figure;
plot(SOC_V, V_data, 'o', 'DisplayName', 'Original Data');
hold on;
plot(SOC_range_V, Voltage_fit, 'r-', 'DisplayName', '2nd-degree Polynomial Fit');
xlabel('State of Charge (SOC)');
ylabel('Voltage');
title('2nd-degree Polynomial Fit: Voltage as a function of SOC');
legend('Location', 'Best');
grid on;

%% Resultant polynomial functions

% EM polynomial fit

function eff = eff_poly(w, T)
    % eff = 2.07338239e-6*T - 3.84695020e-6*w + 8.63478964e-1;
    e = 3.86559103e-6*T^2 -1.15468531e-9*w^2 - 6.64654450e-9*T*w + 3.14229679e-5*T + 1.63013222e-5*w + 7.71598709e-1;
    if e > 1
        eff = 1;
    elseif e < 0
        eff = 0;
    else
        eff = e;
    end
end

% Battery resistance polynomial fit

function V_oc = V_poly(SOC)
    V_oc = -0.40666*SOC^2 + 1.0703*SOC + 3.4385;
end

% Battery voltage polynomial fit

function R_o = R_poly(SOC)
    R_o = 0.00041627*SOC^2 - 0.00071804*SOC + 0.0023018;
end