clear;
clc;
close all;

% testing polinomial fit with battery and EM efficiency data

%% adding folders to path
addpath('..\data\Eff_map');
addpath('..\data');

%% loading data into vectors

%%% loading EM parameters
load('Efficiency.mat'); % loading the EM efficeincy map
load("Shaft_torque.mat"); % loading shaft torque map
load("Speed.mat"); % loading speed map 

%%% loading battery params
load("bat_Ro_vs_SOC_data.mat");
load("bat_Voc_vs_SOC_data.mat");

%% EM efficiency polynomial polynomial fitting and testing

% EM efficiency ref (compare the results with the polinomial)
Efficiency(Efficiency == 0) = 10;
eff_flat = Efficiency(:)/100;
torq_flat = Shaft_Torque(:);
speed_flat = Speed(:);

F = scatteredInterpolant(Speed(:),Shaft_Torque(:),Efficiency(:)/100);


speed_points = [500;750;1000;1250;1500;1750;2000;2250;2500;2750;3000;3250;
    3500;3750;4000;4250;4500;4750;5000;5250;5500;5750;6000;6250;6500;6750;
    7000;7250;7500;7750;8000;8250;8500;8750;9000;9250;9500;9750;10000];


torque_points = [287.315725265347;287.233717032146;287.151724602149;
    287.069743739971;286.987771986552;286.905807734913;286.823849853203;
    286.741897501612;267.378269069311;243.071146629631;222.814866998334;
    205.674858958375;190.983337167297;178.250632202600;167.109517497491;
    157.278993072683;148.540714541255;140.726295202330;133.690149986783;
    127.323954482924;121.536502005985;116.252306265388;111.408460170112;
    106.952121758109;102.838578612008;99.0297423697019;95.4928054975261;
    92.1999538366269;89.1266218638583;86.2515686389848;83.5562072631545;
    81.0242030664065;78.6411424126642;76.3942584067584;74.2722022887362;
    72.2648518732074;70.3631501887848;68.5589689496418;66.8449924891107];

%%% testing and comparing the results of polynimial fit and interpolant 

poly = [];
eval = [];
number_points = [];
for i=1:length(speed_points)
    poly = [poly; eff_poly(speed_points(i),torque_points(i))];
    eval = [eval; F(speed_points(i),torque_points(i))];
    number_points = [number_points;i];    
end

% plotting the diferentece between the evaluations
directory = "C:\Users\gabri\Documents\TCC - EV powertrain control\Text\images";


fig = figure();
fig.Position = [100, 100, 1000, 600];
plot(number_points,poly - eval, 'LineWidth',1.5);
title('Difference from polynomial fit and interpolant evaluation of EM efficiency');
xlabel('Evaluations');
ylabel('Difference');
grid on;

poly_plot = zeros(size(Speed));
eval_plot = zeros(size(Speed));
s = size(Speed);
for i = 1:s(1)
    for j=1:s(2)
        poly_plot(i,j) = eff_poly(Speed(i,j), Shaft_Torque(i,j));
        eval_plot(i,j) = F(Speed(i,j), Shaft_Torque(i,j));
    end
end

Speed_max = Speed(:,1);
Torque_max = Shaft_Torque(:,1);

disp(["Mean absolute error (MAE): " mean_absolute_error(eval_plot(:), poly_plot(:))]);
disp(["Root mean squared error (RMSE): ",root_mean_squared_error(eval_plot(:), poly_plot(:))]);

% electric motor efficiency map - polynomial fit
fig = figure();
fig.Position = [100, 100, 1000, 600];
levels = [0:0.70:0.70 00.70:0.1:0.94 0.94:0.01:1];
M = contourf(Speed,Shaft_Torque,poly_plot,levels);
hold on
plot(Speed_max,Torque_max)
grid on 
title('Polynomial fit efficiency map')
xlabel('EM speed [rpm]')
ylabel('Shaft Torque [Nm]')


filename = 'polynomial_eff_map_2d.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);

fig = figure();
surf(Speed, Shaft_Torque, poly_plot);
title("Plynomial fitted efficiency map");
xlabel('EM speed [rpm]')
ylabel('Shaft Torque [Nm]')
zlabel('Efficiency [-]')
filename = 'polynomial_eff_map_3d.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);

fig = figure();
surf(Speed, Shaft_Torque, eval_plot);
title("Experimental efficiency map");
xlabel('EM speed [rpm]')
ylabel('Shaft Torque [Nm]')
zlabel('Efficiency [-]')
zlim([0.6,1]);
filename = 'z_limited_experimental_eff_map_3d.jpg';
fullFileName = fullfile(directory, filename);
saveas(fig, fullFileName);


%% Battery voltage and resistance polynomial fitting and testing

% interpolants for comparison and tests
F_Voc_s = griddedInterpolant(SOC_Voc_data(:,1),SOC_Voc_data(:,2)); % interpolated Voc data for a single cell
F_Ro_s = griddedInterpolant(SOC_Ro_data(:,1),SOC_Ro_data(:,2)); % interpolated Ro data for a single cell

% Extracting data from files
SOC_R = SOC_Ro_data(:,1); 
R_data = SOC_Ro_data(:,2);

SOC_V = SOC_Voc_data(:,1);
V_data = SOC_Voc_data(:,2);

% Perform a 2nd-degree polynomial fit
degree = 2;
R_coefficients = polyfit(SOC_R, R_data, degree);
V_coefficients = polyfit(SOC_V, V_data, degree);

% Polynomial functions
R_function = poly2str(R_coefficients, 'SOC');
V_function = poly2str(V_coefficients, 'SOC');
disp(['Polynomial Equation: R(SOC) = ' R_function]);
disp(['Polynomial Equation: V(SOC) = ' V_function]);

% Create a polynomial model using polyval
SOC_range_R = linspace(min(SOC_R), max(SOC_R), 100); % Adjust the range as needed
Resistance_fit = polyval(R_coefficients, SOC_range_R);

SOC_range_V = linspace(min(SOC_V), max(SOC_V), 100); % Adjust the range as needed
Voltage_fit = polyval(V_coefficients, SOC_range_V);

% Plot the original data points and the fitted curve
% figure;
% plot(SOC_R, R_data, 'o', 'DisplayName', 'Original Data');
% hold on;
% plot(SOC_range_R, Resistance_fit, 'r-', 'DisplayName', '2nd-degree Polynomial Fit');
% xlabel('State of Charge (SOC)');
% ylabel('Resistance');
% title('2nd-degree Polynomial Fit: Resistance as a function of SOC');
% legend('Location', 'Best');
% grid on;
% 
% 
% figure;
% plot(SOC_V, V_data, 'o', 'DisplayName', 'Original Data');
% hold on;
% plot(SOC_range_V, Voltage_fit, 'r-', 'DisplayName', '2nd-degree Polynomial Fit');
% xlabel('State of Charge (SOC)');
% ylabel('Voltage');
% title('2nd-degree Polynomial Fit: Voltage as a function of SOC');
% legend('Location', 'Best');
% grid on;




%% Resultant polynomial functions

% EM polynomial fit

function eff = eff_poly(w, T)
    e = 0.94959028507595 + -0.00011142427627*w + 0.00001610889481*T + 0.00000002053221*w^2 + -0.00000000873770*w*T + -0.00000505409914*T^2 + -0.00000000000114*w^3 + 0.00000000000073*w^2*T + 0.00000000452099*w*T^2 + 0.00000000028660*T^3;
    if e > 1
        eff = 1;
    elseif e < 0
        eff = 0;
    else
        eff = e;
    end
end

% Battery resistance polynomial fit

function V_oc = V_poly(SOC)
    V_oc = -0.40666*SOC^2 + 1.0703*SOC + 3.4385;
end

% Battery voltage polynomial fit

function R_o = R_poly(SOC)
    R_o = 0.00041627*SOC^2 - 0.00071804*SOC + 0.0023018;
end


function mae = mean_absolute_error(y_true, y_pred)
    mae = mean(abs(y_true - y_pred));
end

function rmse = root_mean_squared_error(y_true, y_pred)
    rmse = sqrt(mean((y_true - y_pred).^2));
end
