clc
clear
close all

% enviroment parameters
g = 9.81; % Gravity acceleration [m/s^2]
alpha = 0*pi/180; % road slope [rad]
rho = 1.25; % air density [kg/m^3] - from Onori HEV book

% vehicle parameters
M_veh = 1400; % vehicle mass [kg]
f_0 = 4.5/1000; % Static rolling coefficient [N/kN]
k = 0; % miscellaneous loss coeff [Ns/m]
r_w = 0.3; % Wheel radius [m]
C_d = 0.33; % Drag coeff
A_f = 2.15; % Frontal area [m^2]
tau_gb = 9.6; % Gear ratio
eta_gb = 0.97; % Gearbox efficiency
eta_inv = 1; % Inverter efficiency
phi = r_w/tau_gb;


% battery parameters
N_s = 108; % Number of series
N_p = 1; % Number of parallels
N_b = N_s*N_p; % number of battery cells
Q_nom = 60*N_p*3600; % nominal battery capacity [As]
eta_batt = 0.95; % Coloumbic efficiency

% WLTP 3 
load(fullfile('data','WLTP3.mat')) % loading the speed profile (WLTP3)
% velocity and acceleration profiles
vehspeed_prof = [time_s,speed_kmh/3.6]; % speed profile in [m/s]
vehspeed = [speed_kmh/3.6];
dt = 1; 
vehacc = (vehspeed_prof(2:end,2)-vehspeed_prof(1:end-1,2))./dt; %vehicle acceleration [m/s^2]
vehacc = [0; vehacc];

% Forces 
F_roll = M_veh*g*f_0; % rolling resistance force
K_aero = 0.5*C_d*rho*A_f; % aerodynamic constant
% F_aero = K_aero * x(3)^2;

% EM equations
% w_EM = x(3)/phi; 
% w_EM_rpm = w_EM*30/pi;
% eta_EM = eff_poly(w_EM_rpm, T_EM);
% P_EM = w_EM*T_EM;

% battery equations
% Pb = P_EM/((eta_EM*eta_inv)^(sign(P_EM)));
% V_oc = N_s*V_poly(x1); % vector with interpolated values of Voc_s (single cell)
% R_o = (N_s/N_p)*R_poly(x1); % vector with interpolated values of Ro_s (single cell)
% Ib = (V_oc - sqrt(V_oc^2 - 4*R_o*Pb))/(2*R_o);

% Ib = (N_s*V_poly(x(1)) - sqrt((N_s*V_poly(x(1)))^2 - 4*N_s*R_poly(x(1))*x(3)*u/(phi*N_p*(eta_inv*eff_poly(x(3)*30/(pi*phi),u))^(sign(x(3)*u)))))/(2*(N_s/N_p)*R_poly(x(1)));

% state space equations
% soc_dot = -((N_s*V_poly(x(1)) - sqrt((N_s*V_poly(x(1)))^2 - 4*N_s*R_poly(x(1))*x(3)*u/(phi*N_p*(eta_inv*eff_poly(x(3)*30/(pi*phi),u))^(sign(x(3)*u)))))/(2*(N_s/N_p)*R_poly(x(1))))*eta_batt^(-sign((N_s*V_poly(x(1)) - sqrt((N_s*V_poly(x(1)))^2 - 4*N_s*R_poly(x(1))*x(3)*u/(phi*N_p*(eta_inv*eff_poly(x(3)*30/(pi*phi),u))^(sign(x(3)*u)))))/(2*(N_s/N_p)*R_poly(x(1)))))/Q_nom;
% x_dot = x(3);
% v_dot = ((eta_gb/phi)*u - F_roll - K_aero*(x(3)^2))/M_veh;
func = @(t, x, u) [-((N_s*V_poly(x(1)) - sqrt((N_s*V_poly(x(1)))^2 - 4*N_s*R_poly(x(1))*x(3)*u/(phi*N_p*(eta_inv*eff_poly(x(3)*30/(pi*phi),u))^(sign(x(3)*u)))))/(2*(N_s/N_p)*R_poly(x(1))))*eta_batt^(-sign((N_s*V_poly(x(1)) - sqrt((N_s*V_poly(x(1)))^2 - 4*N_s*R_poly(x(1))*x(3)*u/(phi*N_p*(eta_inv*eff_poly(x(3)*30/(pi*phi),u))^(sign(x(3)*u)))))/(2*(N_s/N_p)*R_poly(x(1)))))/Q_nom;
                 x(3);
                ((eta_gb/phi)*u - F_roll - K_aero*(x(3)^2))/M_veh];

% Time span
tspan = [0 430];

% Input signal
% t_input = linspace(0,1000,1000); % Time points for input
t_input = time_s;
U = torque_input(vehacc, vehspeed); % Example input signal, you can define your own
% U = 250*sin(t_input*0.2);

% Interpolation of input signal
U_interp = @(t) interp1(t_input, U, t);


% testing prediction model
SOC0 = 0.8;
x0 = 0;
v0 = 0;
initialState = [SOC0; x0; v0];
% initialState = [x0; v0];

% Function for ODE solver
odefun = @(t, x) prediction_longitudinal_model(t, x, U_interp(t));

% Solve the system using ode45
[t, x] = ode45(odefun, tspan, initialState);
    
% Plot results
figure;
subplot(3, 1, 1);
plot(t, x(:, 1), 'b','LineWidth',1.5) %, t, x(:, 3),'b');
xlabel('Time [s]');
ylabel('SOC [-]');
title('State X1 - Battery State Of Charge');
grid on;

subplot(3, 1, 2);
plot(t, x(:, 2), 'g','LineWidth',1.5) %, t, x(:, 3),'b');
xlabel('Time');
ylabel('State Variables');
title('State X2 - Vehicle Position');
grid on;

subplot(3, 1, 3);
plot(t, x(:, 3), 'r', 'LineWidth',1.5) 
xlabel('Time');
ylabel('State Variables');
title('State X3 - Vehicle Velocity');
grid on;

figure;
plot(t_input, U, 'k','LineWidth',1.5);
xlabel('Time [s]');
ylabel('Input (U) - T_{EM} [Nm]');
title('Input Signal');
xlim([0 tspan(2)]);
grid on;


function T_EM = torque_input(a_ref, v_ref)

    % enviroment parameters
    g = 9.81; % Gravity acceleration [m/s^2]
    alpha = 0*pi/180; % road slope [rad]
    rho = 1.25; % air density [kg/m^3] - from Onori HEV book
    
    % vehicle parameters
    M_veh = 1400; % vehicle mass [kg]
    f_0 = 4.5/1000; % Static rolling coefficient [N/kN]
    k = 0; % miscellaneous loss coeff [Ns/m]
    r_w = 0.3; % Wheel radius [m]
    C_d = 0.33; % Drag coeff
    A_f = 2.15; % Frontal area [m^2]
    tau_gb = 9.6; % Gear ratio
    eta_gb = 0.97; % Gearbox efficiency
    
    F_intertia = M_veh*a_ref;
    F_roll = M_veh*f_0*g;
    F_misc = M_veh*g*sin(alpha);
    F_aero = 0.5*rho*C_d*A_f*v_ref.^2;
    
    T_wheel = (F_intertia+F_roll+F_aero+F_misc)*r_w;

    T_EM = T_wheel./(tau_gb*eta_gb.^(sign(T_wheel)));
end
